@@ -41,15 +41,20 @@
 #define NM_LIBRESWAN_KEY_LEFTID                     "leftid"
 #define NM_LIBRESWAN_KEY_LEFTRSASIGKEY              "leftrsasigkey"
 #define NM_LIBRESWAN_KEY_LEFTCERT                   "leftcert"
+#define NM_LIBRESWAN_KEY_AUTHBY                     "authby"
 #define NM_LIBRESWAN_KEY_PSK_VALUE                  "pskvalue"
 #define NM_LIBRESWAN_KEY_PSK_INPUT_MODES            "pskinputmodes"
 #define NM_LIBRESWAN_KEY_LEFTXAUTHUSER              "leftxauthusername"
+#define NM_LIBRESWAN_KEY_LEFTUSERNAME               "leftusername"
 #define NM_LIBRESWAN_KEY_XAUTH_PASSWORD             "xauthpassword"
 #define NM_LIBRESWAN_KEY_XAUTH_PASSWORD_INPUT_MODES "xauthpasswordinputmodes"
 #define NM_LIBRESWAN_KEY_DOMAIN                     "Domain"
 #define NM_LIBRESWAN_KEY_DHGROUP                    "dhgroup"
+#define NM_LIBRESWAN_KEY_PFS                        "pfs"
 #define NM_LIBRESWAN_KEY_PFSGROUP                   "pfsgroup"
 #define NM_LIBRESWAN_KEY_DPDTIMEOUT                 "dpdtimeout"
+#define NM_LIBRESWAN_KEY_DPDDELAY                   "dpddelay"
+#define NM_LIBRESWAN_KEY_DPDACTION                  "dpdaction"
 #define NM_LIBRESWAN_KEY_IKE                        "ike"
 #define NM_LIBRESWAN_KEY_ESP                        "esp"
 #define NM_LIBRESWAN_KEY_IKELIFETIME                "ikelifetime"
@@ -61,6 +66,7 @@
 #define NM_LIBRESWAN_KEY_REKEY                      "rekey"
 #define NM_LIBRESWAN_KEY_FRAGMENTATION              "fragmentation"
 #define NM_LIBRESWAN_KEY_MOBIKE                     "mobike"
+#define NM_LIBRESWAN_KEY_IPSEC_INTERFACE            "ipsec-interface"
 
 #define NM_LIBRESWAN_IKEV2_NO      "no"
 #define NM_LIBRESWAN_IKEV2_NEVER   "never"
@@ -72,6 +78,10 @@
 #define NM_LIBRESWAN_PW_TYPE_ASK    "ask"
 #define NM_LIBRESWAN_PW_TYPE_UNUSED "unused"
 
+#define NM_LIBRESWAN_AGGRMODE_DEFAULT_IKE   "aes256-sha1;modp1536"
+#define NM_LIBRESWAN_AGGRMODE_DEFAULT_ESP   "aes256-sha1"
+#define NM_LIBRESWAN_IKEV1_DEFAULT_LIFETIME "24h"
+
 #ifndef NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV_NONE
 #define NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV_NONE "_none_"
 #endif
