@@ -106,6 +106,7 @@ typedef struct {
 	gboolean pending_auth;
 	gboolean managed;
 	gboolean xauth_enabled;
+	int version;
 
 	GPid pid;
 	guint watch_id;
@@ -252,13 +253,17 @@ static ValidProperty valid_properties[] = {
 	{ NM_LIBRESWAN_KEY_LEFT,                       G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_LEFTID,                     G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_LEFTXAUTHUSER,              G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_LEFTUSERNAME,               G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_LEFTRSASIGKEY,              G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_LEFTCERT,                   G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_AUTHBY,                     G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_DOMAIN,                     G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_DHGROUP,                    G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_PFS,                        G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_PFSGROUP,                   G_TYPE_STRING, 0, 0 },
-	{ NM_LIBRESWAN_KEY_DPDTIMEOUT,                 G_TYPE_INT, 0, 86400 },
+	{ NM_LIBRESWAN_KEY_DPDACTION,                  G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_DPDDELAY,                   G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_DPDTIMEOUT,                 G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_IKE,                        G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_ESP,                        G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_IKELIFETIME,                G_TYPE_STRING, 0, 0 },
@@ -270,6 +275,7 @@ static ValidProperty valid_properties[] = {
 	{ NM_LIBRESWAN_KEY_REKEY,                      G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_FRAGMENTATION,              G_TYPE_STRING, 0, 0 },
 	{ NM_LIBRESWAN_KEY_MOBIKE,                     G_TYPE_STRING, 0, 0 },
+	{ NM_LIBRESWAN_KEY_IPSEC_INTERFACE,            G_TYPE_STRING, 0, 0 },
 	/* Ignored option for internal use */
 	{ NM_LIBRESWAN_KEY_PSK_INPUT_MODES,            G_TYPE_NONE, 0, 0 },
 	{ NM_LIBRESWAN_KEY_XAUTH_PASSWORD_INPUT_MODES, G_TYPE_NONE, 0, 0 },
@@ -413,68 +419,6 @@ unblock_quit (NMLibreswanPlugin *self)
 
 static gboolean connect_step (NMLibreswanPlugin *self, GError **error);
 
-static const char *
-_find_helper (const char *progname, const char **paths, GError **error)
-{
-	const char **iter = paths;
-	GString *tmp;
-	const char *ret = NULL;
-
-	if (error)
-		g_return_val_if_fail (*error == NULL, NULL);
-
-	tmp = g_string_sized_new (50);
-	for (iter = paths; iter && *iter; iter++) {
-		g_string_append_printf (tmp, "%s%s", *iter, progname);
-		if (g_file_test (tmp->str, G_FILE_TEST_EXISTS)) {
-			ret = g_intern_string (tmp->str);
-			break;
-		}
-		g_string_set_size (tmp, 0);
-	}
-	g_string_free (tmp, TRUE);
-
-	if (!ret) {
-		g_set_error (error, NM_VPN_PLUGIN_ERROR, NM_VPN_PLUGIN_ERROR_LAUNCH_FAILED,
-		             "Could not find %s binary",
-		             progname);
-	}
-	return ret;
-}
-
-static const char *
-find_helper_bin (const char *progname, GError **error)
-{
-	static const char *paths[] = {
-		PREFIX "/sbin/",
-		PREFIX "/bin/",
-		"/sbin/",
-		"/usr/sbin/",
-		"/usr/local/sbin/",
-		"/usr/bin/",
-		"/usr/local/bin/",
-		NULL,
-	};
-
-	return _find_helper (progname, paths, error);
-}
-
-static const char *
-find_helper_libexec (const char *progname, GError **error)
-{
-	static const char *paths[] = {
-		PREFIX "/libexec/ipsec/",
-		PREFIX "/lib/ipsec/",
-		"/usr/libexec/ipsec/",
-		"/usr/local/libexec/ipsec/",
-		"/usr/lib/ipsec/",
-		"/usr/local/lib/ipsec/",
-		NULL,
-	};
-
-	return _find_helper (progname, paths, error);
-}
-
 static void
 connect_cleanup (NMLibreswanPlugin *self)
 {
@@ -801,7 +745,7 @@ spawn_pty (NMLibreswanPlugin *self,
            const char *progname,
            ...)
 {
-	int pty_master_fd = 0;
+	int pty_controller_fd = 0;
 	int stdout_pipe[2], stderr_pipe[2];
 	pid_t child_pid;
 	struct termios termios_flags;
@@ -903,7 +847,7 @@ spawn_pty (NMLibreswanPlugin *self,
 	_LOGI ("PTY spawn: %s", (cmdline = g_strjoinv (" ", (char **) argv->pdata)));
 
 	/* Fork the command */
-	child_pid = forkpty (&pty_master_fd, NULL, NULL, NULL);
+	child_pid = forkpty (&pty_controller_fd, NULL, NULL, NULL);
 	if (child_pid == 0) {
 		/* in the child */
 
@@ -951,17 +895,17 @@ spawn_pty (NMLibreswanPlugin *self,
 	}
 	g_ptr_array_free (argv, TRUE);
 
-	tcgetattr (pty_master_fd, &termios_flags);
+	tcgetattr (pty_controller_fd, &termios_flags);
 	cfmakeraw (&termios_flags);
 	cfsetospeed (&termios_flags, __MAX_BAUD);
-	tcsetattr (pty_master_fd, TCSANOW, &termios_flags);
+	tcsetattr (pty_controller_fd, TCSANOW, &termios_flags);
 
 	if (out_stdout)
 		*out_stdout = stdout_pipe[0];
 	if (out_stderr)
 		*out_stderr = stderr_pipe[0];
 	if (out_ptyin)
-		*out_ptyin = pty_master_fd;
+		*out_ptyin = pty_controller_fd;
 	if (out_pid)
 		*out_pid = child_pid;
 
@@ -1254,7 +1198,7 @@ _take_route (GPtrArray *routes, GVariant *new, gboolean alive)
 }
 
 static void
-handle_route (GPtrArray *routes, GVariant *env, gboolean alive)
+handle_route (GPtrArray *routes, GVariant *env, gboolean alive, gboolean is_xfrmi)
 {
 	GVariantBuilder builder;
 	const gchar *net, *mask, *next_hop, *my_sourceip;
@@ -1267,9 +1211,13 @@ handle_route (GPtrArray *routes, GVariant *env, gboolean alive)
 	next_hop = lookup_string (env, "PLUTO_NEXT_HOP");
 	my_sourceip = lookup_string (env, "PLUTO_MY_SOURCEIP");
 
+
 	if (!net || !mask || !next_hop || !my_sourceip)
 		return;
 
+	if (is_xfrmi)
+		next_hop = "0.0.0.0";
+
 	if (g_strcmp0 (net, "0.0.0.0") == 0 && g_strcmp0 (mask, "0")) {
 		g_variant_builder_init (&builder, G_VARIANT_TYPE ("au"));
 		g_variant_builder_add_value (&builder, addr4_to_gvariant ("0.0.0.0"));
@@ -1310,6 +1258,8 @@ handle_callback (NMDBusLibreswanHelper *object,
 	gboolean success = FALSE;
 	guint i;
 	const char *verb;
+	const char *virt_if;
+	gboolean is_xfrmi = FALSE;
 
 	_LOGI ("Configuration from the helper received.");
 
@@ -1332,9 +1282,14 @@ handle_callback (NMDBusLibreswanHelper *object,
 
 	/*
 	 * Tunnel device
-	 * Indicate that this plugin doesn't use tun/tap device
 	 */
-	val = g_variant_new_string (NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV_NONE);
+	virt_if = lookup_string (env, "PLUTO_VIRT_INTERFACE");
+	if (virt_if && !nm_streq (virt_if, "NULL")) {
+		val = g_variant_new_string (virt_if);
+	} else {
+		val = g_variant_new_string (NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV_NONE);
+	}
+
 	g_variant_builder_add (&config, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_TUNDEV, val);
 
 	/* IP address */
@@ -1383,16 +1338,23 @@ handle_callback (NMDBusLibreswanHelper *object,
 	if (val)
 		g_variant_builder_add (&config, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_BANNER, val);
 
+	/* Indicates whether the VPN is using a XFRM interface (via option ipsec-interface=) */
+	is_xfrmi = nm_streq0 (lookup_string (env, "PLUTO_XFRMI_ROUTE"), "yes");
 
-	val = addr4_to_gvariant (lookup_string (env, "PLUTO_NEXT_HOP"));
-	if (val)
-		g_variant_builder_add (&config, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_INT_GATEWAY, val);
+	if (is_xfrmi) {
+		/* The traffic needs to be sent directly over the interface without a gateway.
+		 * Ignore the next hop. */
+	} else {
+		val = addr4_to_gvariant (lookup_string (env, "PLUTO_NEXT_HOP"));
+		if (val)
+			g_variant_builder_add (&config, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_INT_GATEWAY, val);
+	}
 
 	/* This route */
 	if (g_strcmp0 (verb, "route-client") == 0 || g_strcmp0 (verb, "route-host"))
-		handle_route (priv->routes, env, TRUE);
+		handle_route (priv->routes, env, TRUE, is_xfrmi);
 	else if (g_strcmp0 (verb, "unroute-client") == 0 || g_strcmp0 (verb, "unroute-host"))
-		handle_route (priv->routes, env, FALSE);
+		handle_route (priv->routes, env, FALSE, is_xfrmi);
 
 	/* Routes */
 	g_variant_builder_init (&builder, G_VARIANT_TYPE ("aau"));
@@ -1578,7 +1540,7 @@ connect_step (NMLibreswanPlugin *self, GError **error)
 		if (!priv->openswan) {
 			const char *stackman_path;
 
-			stackman_path = find_helper_libexec ("_stackmanager", error);
+			stackman_path = nm_libreswan_find_helper_libexec ("_stackmanager", error);
 			if (!stackman_path)
 				return FALSE;
 
@@ -1650,6 +1612,7 @@ connect_step (NMLibreswanPlugin *self, GError **error)
 		                                   bus_name);
 
 		if (!nm_libreswan_config_write (fd,
+		                                priv->version,
 		                                priv->connection,
 		                                uuid,
 		                                ifupdown_script,
@@ -1695,20 +1658,6 @@ connect_step (NMLibreswanPlugin *self, GError **error)
 	g_assert_not_reached ();
 }
 
-static gboolean
-is_openswan (const char *path)
-{
-	const char *argv[] = { path, NULL };
-	gboolean openswan = FALSE;
-	char *output = NULL;
-
-	if (g_spawn_sync (NULL, (char **) argv, NULL, 0, NULL, NULL, &output, NULL, NULL, NULL)) {
-		openswan = output && strcasestr (output, " Openswan ");
-		g_free (output);
-	}
-	return openswan;
-}
-
 static gboolean
 _connect_common (NMVpnServicePlugin   *plugin,
                  NMConnection  *connection,
@@ -1719,22 +1668,26 @@ _connect_common (NMVpnServicePlugin   *plugin,
 	NMLibreswanPluginPrivate *priv = NM_LIBRESWAN_PLUGIN_GET_PRIVATE (self);
 	NMSettingVpn *s_vpn;
 	const char *con_name = nm_connection_get_uuid (connection);
+	gs_free char *ipsec_banner = NULL;
 
 	if (_LOGD_enabled ()) {
 		_LOGD ("connection:");
 		nm_connection_dump (connection);
 	}
 
-	priv->ipsec_path = find_helper_bin ("ipsec", error);
+	priv->ipsec_path = nm_libreswan_find_helper_bin ("ipsec", error);
 	if (!priv->ipsec_path)
 		return FALSE;
 
-	priv->openswan = is_openswan (priv->ipsec_path);
+	nm_libreswan_detect_version (priv->ipsec_path, &priv->openswan, &priv->version, &ipsec_banner);
+	_LOGD ("ipsec: version banner: %s", ipsec_banner);
+	_LOGD ("ipsec: detected version %d (%s)", priv->version, priv->openswan ? "Openswan" : "Libreswan");
+
 	if (!priv->openswan) {
-		priv->pluto_path = find_helper_libexec ("pluto", error);
+		priv->pluto_path = nm_libreswan_find_helper_libexec ("pluto", error);
 		if (!priv->pluto_path)
 			return FALSE;
-		priv->whack_path = find_helper_libexec ("whack", error);
+		priv->whack_path = nm_libreswan_find_helper_libexec ("whack", error);
 		if (!priv->whack_path)
 			return FALSE;
 	}
